import re
import urllib

import requests
from bs4 import BeautifulSoup

from bkmrks import urls


def get_url_icon(url):

    domain = urls.extract_domain_from_url(url=url)
    if len(domain) == 0:
        return get_default_img(text=url)

    meta_icon, url_soup = get_meta_icon_from_url(url=url)
    if meta_icon is not None:
        return meta_icon

    favicon = get_favicon_from_url(url=url)
    if favicon is not None:
        return favicon

    url_first_img = get_first_img_from_url(url=url, url_soup=url_soup)
    if url_first_img is not None:
        return urls.ensure_domain(url_first_img, domain)

    domain_first_img = get_first_img_from_url(url=domain)
    if domain_first_img is not None:
        return urls.ensure_domain(domain_first_img, domain)

    return get_default_img(text=url)


def get_meta_icon_from_url(url, url_soup=None):
    soup_icons, url_soup = get_soup_icons_from_url(url, url_soup)

    if soup_icons is None or len(soup_icons) == 0:
        img = None
        return img, url_soup
    else:
        final_icon = soup_icons[0]
        for soup_icon in soup_icons:
            if get_soup_icon_size(soup_icon) >= get_soup_icon_size(final_icon):
                final_icon = soup_icon
        img = final_icon["href"]

    img = urls.ensure_relative_path(img, url)
    return img, url_soup


def get_soup_icons_from_url(url, url_soup=None):
    if url_soup is None:
        try:
            url_request = requests.get(url)
            url_soup = BeautifulSoup(url_request.text, features="html.parser")
        except:
            return None, None

    soup_icons = url_soup.find_all(
        "link",
        attrs={
            "rel": [
                "icon",
                "apple-touch-icon",
            ]
        },
    )
    return soup_icons, url_soup


def get_soup_icon_size(soup_icon):
    if not soup_icon.has_attr("sizes"):
        return 1
    return int(soup_icon["sizes"].split("x")[0])


def get_favicon_from_url(url):
    domain = urls.extract_domain_from_url(url=url)
    favicon = domain + "/favicon.ico"
    try:
        favicon_request = requests.get(favicon)
    except:
        return None

    if favicon_request.status_code == 404 or "html" in favicon_request.text[:150]:
        return None
    else:
        return favicon


def get_first_img_from_url(url, url_soup=None):
    try:
        url_request = requests.get(url)
    except:
        return None

    if url_request.status_code == 404:
        return None
    if url_soup is None:
        url_soup = BeautifulSoup(url_request.text, features="html.parser")

    first_img = url_soup.find("img")
    if first_img is not None:
        img = urls.ensure_domain(first_img["src"], url)
        return img
    else:
        return None


def get_default_img(text):
    text = re.sub(r"[^a-zA-Z0-9./+]", "", text)
    text = text.replace("www", "").replace("https", "")
    text = text.replace(".", " ").replace("/", " ").strip()
    text = urllib.parse.quote(text, safe="=?&")
    return f"https://ui-avatars.com/api/?name={text}"


def get_img_from_a_soup_item(soup_item, domain):
    soup_item["href"] = urls.ensure_domain(url=soup_item["href"], domain=domain)

    use_soup_img = False
    if len(soup_item.find_all("img")) > 0:
        if len(soup_item.find("img")["src"]) < 200:
            use_soup_img = False
        else:
            use_soup_img = True

    if use_soup_img:
        img = soup_item.find("img")["src"]
    else:
        img = get_url_icon(soup_item["href"])
    return img



from bkmrks import md

def test_md_hr():
    assert "\n---" == md.md_hr()

def test_md_a_img():
    tests = []

    url = "https://testing.url"
    name = "name_testing"
    img = ""
    item = {"url":url, "name":name, "img":img, }

    expected_result = f'\n[![{name}]({img})]({url} "{name}")'

    tests.append({"item":item, "expected_result":expected_result})

    name = ""
    img = ""
    item = {"name":name, "img":img, }
    expected_result = ""
    tests.append({"item":item, "expected_result":""})

    item = ""
    expected_result = ""
    tests.append({"item":item, "expected_result":""})

    url = "https://testing.url"
    name = "name_testing"
    img = ""
    item = {"url":url, "name":name, "img":img, }

    expected_result = f'\n[![{name}]({img})]({url} "{name}")'

    tests.append({"item":item, "expected_result":expected_result})

    for test in tests:
        assert test["expected_result"] == md.md_a_img(item=test["item"])



import os

import markdown

from bkmrks import bkmrks, files, folders, md


def render():
    bkmrks.clean_catalogs()
    clean_public_folder()
    bookmarks = os.listdir(folders.catalogs_folder())
    menu = []
    htmls = []
    mds = []
    for catalog in bookmarks:
        md_file_name = folders.public_folder(path=catalog)
        md_generated_file = md.generate(md_file_name=md_file_name, catalog=catalog)
        mds.append(md_generated_file)
        if md_generated_file is not None:
            htmls.append(generate_html(md_file_name))
            menu.append(
                get_file_and_set_variable(
                    file=folders.templates_folder(path="menu_item.html"),
                    variable="menu_item",
                    content=files.extract_file_name_no_ext(catalog),
                )
            )
    menu = " | ".join(menu)
    for html in htmls:
        html_content = get_file_and_set_variable(
            file=html,
            variable="menu",
            content=menu,
        )
        with open(html, "+w") as f:
            f.write(html_content)

    delete_files(files=mds)


def generate_html(md_file=None, template="index"):
    if md_file is None:
        md_file = folders.public_folder(path="index")

    md_file = files.apply_ext(md_file, ext="md")

    with open(md_file, "r") as fm:
        html = set_template_content(
            markdown.markdown(fm.read()), template, extension="html"
        )

        catalog = files.extract_file_name_no_ext(file_path=md_file)
        html = html.replace("{__catalog__}", catalog)

        html_file_name = files.apply_ext(file_path=md_file, ext="html")
        with open(html_file_name, "+w") as fh:
            fh.write(html)
    return html_file_name


def get_file_and_set_variable(file, variable, content):
    with open(file, "r") as f:
        file_content = f.read().replace("{" + variable + "}", content)
    return file_content


def get_template(base_file_name, extension="html"):

    template = "{" + extension + "}"
    path = files.apply_ext(base_file_name, extension)

    template_file = folders.templates_folder(path=path)
    if os.path.exists(template_file):
        with open(template_file, "r") as f:
            template = f.read()

        if extension == "html":
            dirs = os.listdir(folders.templates_folder())
            for file in dirs:
                if files.extract_ext(file) != "html" and files.extract_file_name_no_ext(
                    file
                ) == files.extract_file_name_no_ext(base_file_name):
                    innerextension = files.extract_ext(file)
                    innerfile = folders.templates_folder(path=file)
                    with open(innerfile, "r") as f:
                        innertemplate = f.read()
                        template = template.replace(
                            "{" + innerextension + "}", innertemplate
                        )

    return template


def set_template_content(content, base_file_name, extension="html"):
    template = get_template(base_file_name, extension=extension)
    return template.replace("{" + extension + "}", content)


def clean_public_folder():
    public_folder = folders.public_folder()
    public_folder_files = os.listdir(public_folder)
    delete_files(files=public_folder_files, folder=public_folder)


def delete_files(files: list, folder=""):
    for file in files:
        try:
            file = os.path.join(folder, file)
            if os.path.basename(file) != "":
                os.remove(file)
        except:
            pass
